@@ -1,8 +1,11 @@
+# -*- coding: utf-8 -*-
 """igcommit - Checks on Git commits
 
 Copyright (c) 2016, InnoGames GmbH
 """
 
+from __future__ import unicode_literals
+
 from igcommit.base_check import BaseCheck, Severity
 from igcommit.git import Commit
 
@@ -26,21 +29,40 @@ class CommitCheck(BaseCheck):
 
 class CheckCommitMessage(CommitCheck):
     def get_problems(self):
-        for line_id, line in enumerate(self.commit.get_message().splitlines()):
-            if line_id == 1 and line:
-                yield Severity.ERROR, 'summary extends the first line'
-            if line and line[-1] == ' ':
-                yield (
-                    Severity.ERROR,
-                    'line {}: trailing space'.format(line_id + 1)
-                )
-            if line_id > 1 and line.startswith('    ') or line.startswith('>'):
+        for line_id, line in enumerate(self.commit.get_message_lines()):
+            if line_id == 0:
                 continue
-            if len(line) > 72:
-                yield (
-                    Severity.WARNING,
-                    'line {}: longer than 72'.format(line_id + 1)
-                )
+            elif line_id == 1:
+                if line.strip():
+                    yield Severity.ERROR, 'no single line commit summary'
+            else:
+                if line.startswith('    ') or line.startswith('>'):
+                    continue
+
+            if line:
+                for problem in self.get_line_problems(line_id + 1, line):
+                    yield problem
+
+    def get_line_problems(self, line_number, line):
+        if line.rstrip() != line:
+            line = line.rstrip()
+            yield (
+                Severity.ERROR,
+                'line {}: trailing space'.format(line_number)
+            )
+
+        if line.lstrip() != line:
+            line = line.lstrip()
+            yield (
+                Severity.WARNING,
+                'line {}: leading space'.format(line_number)
+            )
+
+        if len(line) > 72:
+            yield (
+                Severity.WARNING,
+                'line {}: longer than 72'.format(line_number)
+            )
 
 
 class CheckCommitSummary(CommitCheck):
@@ -55,6 +77,7 @@ class CheckCommitSummary(CommitCheck):
         'SECURITY',
         'STYLE',
         'TASK',
+        'TEMP',
         'WIP',
         '!!',
     }
@@ -73,20 +96,13 @@ class CheckCommitSummary(CommitCheck):
                 yield problem
             return
 
-        category_index = rest[:24].find(': ')
-        rest_index = category_index + len(': ')
-        if category_index >= 0 and len(rest) > rest_index:
-            for problem in self.get_category_problems(rest[:category_index]):
-                yield problem
-            rest = rest[rest_index:]
-
         for problem in self.get_summary_problems(rest):
             yield problem
 
     def get_revert_commit_problems(self, rest):
         rest = rest[len('Revert'):]
         if not rest.startswith(' "') or not rest.endswith('"'):
-            yield Severity.WARNING, 'ill-formatted revert commit'
+            yield Severity.WARNING, 'ill-formatted revert commit message'
 
     def get_commit_tag_problems(self, tags, rest):
         used_tags = []
@@ -108,35 +124,57 @@ class CheckCommitSummary(CommitCheck):
         if not rest.startswith(' '):
             yield Severity.WARNING, 'commit tags not separated with space'
 
-    def get_category_problems(self, category):
-        if not category[0].isalpha():
-            yield Severity.WARNING, 'commit category starts with non-letter'
-        if category != category.lower():
-            yield Severity.WARNING, 'commit category has upper-case letter'
-        if category[-1] == ' ':
-            yield Severity.WARNING, 'commit category ends with a space'
-
     def get_summary_problems(self, rest):
         if not rest:
-            yield Severity.ERROR, 'no summary'
+            yield Severity.ERROR, 'no commit summary'
             return
 
-        if len(rest) > 50:
-            yield Severity.WARNING, 'summary longer than 50 characters'
+        rest_len = len(rest)
+        if rest_len > 72:
+            yield Severity.ERROR, 'commit summary longer than 72 characters'
+        elif rest_len > 50:
+            yield Severity.WARNING, 'commit summary longer than 50 characters'
 
         if '  ' in rest:
             yield Severity.WARNING, 'multiple spaces'
 
-        if not rest[0].isalpha():
-            yield Severity.WARNING, 'summary start with non-letter'
-        if rest[-1] == '.':
-            yield Severity.WARNING, 'summary ends with a dot'
+        category_index = rest[:24].find(': ')
+        rest_index = category_index + len(': ')
+        if category_index >= 0 and len(rest) > rest_index:
+            for problem in self.get_category_problems(rest[:category_index]):
+                yield problem
+            rest = rest[rest_index:]
+
+        for problem in self.get_title_problems(rest):
+            yield problem
+
+    def get_category_problems(self, category):
+        if not category[0].isalpha():
+            yield Severity.WARNING, 'commit category starts with non-letter'
+        if category.lower() != category:
+            yield Severity.WARNING, 'commit category has upper-case letter'
+        if category.rstrip() != category:
+            yield Severity.WARNING, 'commit category with trailing space'
+
+    def get_title_problems(self, rest):
+        if not rest:
+            yield Severity.ERROR, 'no commit title'
+            return
+
+        first_letter = rest[0]
+        if not first_letter.isalpha():
+            yield Severity.WARNING, 'commit title start with non-letter'
+        elif first_letter.upper() != first_letter:
+            yield Severity.WARNING, 'commit title not capitalized'
+
+        if rest.endswith('.'):
+            yield Severity.WARNING, 'commit title ends with a dot'
 
         first_word = rest.split(' ', 1)[0]
         if first_word.endswith('ed'):
-            yield Severity.WARNING, 'past tense used on summary'
+            yield Severity.WARNING, 'past tense used on commit title'
         if first_word.endswith('ing'):
-            yield Severity.WARNING, 'continuous tense used on summary'
+            yield Severity.WARNING, 'continuous tense used on commit title'
 
 
 class CheckChangedFilePaths(CommitCheck):
