@@ -1,8 +1,11 @@
+# -*- coding: utf-8 -*-
 """igcommit - Configuration of the checks
 
 Copyright (c) 2016, InnoGames GmbH
 """
 
+from __future__ import unicode_literals
+
 from igcommit.commit_list_checks import (
     CheckDuplicateCommitSummaries,
     CheckMisleadingMergeCommit,
