@@ -1,12 +1,16 @@
+# -*- coding: utf-8 -*-
 """igcommit - Checks on files committed to Git
 
 Copyright (c) 2016, InnoGames GmbH
 """
 
+from __future__ import unicode_literals
+
 from re import compile
+from subprocess import Popen, PIPE, STDOUT
 
 from igcommit.base_check import BaseCheck, CheckState, Severity
-from igcommit.git import Commit, CommittedFile, check_returncode
+from igcommit.git import Commit, CommittedFile
 from igcommit.utils import get_exe_path
 
 FILE_EXTENSIONS = {
@@ -58,38 +62,31 @@ class CheckExecutable(CommittedFileCheck):
     to warn about it, as it is very common to omit executable bit on Git.
     However, it would be expensive to look at the content of every file.
     """
-    def prepare(self, obj):
-        new = super(CheckExecutable, self).prepare(obj)
-        if not new or (
-            isinstance(obj, CommittedFile) and not obj.owner_can_execute()
-        ):
-            return None
-        return new
-
     def get_problems(self):
         shebang = self.committed_file.get_shebang()
-        if not shebang:
-            yield Severity.ERROR, 'no shebang'
-            return
+        if self.committed_file.owner_can_execute():
+            if shebang:
+                for problem in self.get_shebang_problems(shebang):
+                    yield problem
 
-        path = shebang.split(None, 1)[0]
-        if not path.startswith('/'):
+                for problem in self.get_exe_problems():
+                    yield problem
+            else:
+                yield Severity.ERROR, 'executable file without shebang'
+        elif shebang:
+            yield Severity.WARNING, 'non-executable file with shebang'
+
+    def get_shebang_problems(self, shebang):
+        if not shebang.startswith('/'):
             yield (
                 Severity.ERROR,
-                'shebang executable {} is not full path'.format(path)
+                'shebang executable {} is not full path'.format(shebang)
             )
-        elif path == '/usr/bin/env':
-            if shebang == path:
-                yield Severity.ERROR, '/usr/bin/env must have an argument'
-                return
-        elif path.startswith('/usr'):
+        elif shebang.startswith('/usr') and shebang != '/usr/bin/env':
             yield (
                 Severity.WARNING, 'shebang is not portable (use /usr/bin/env)'
             )
 
-        for problem in self.get_exe_problems():
-            yield problem
-
     def get_exe_problems(self):
         extension = self.committed_file.get_extension()
         if not extension:
@@ -100,7 +97,10 @@ class CheckExecutable(CommittedFileCheck):
                 yield Severity.WARNING, 'general executable name'
             return
 
-        exe = self.committed_file.get_exe()
+        exe = self.committed_file.get_shebang_exe()
+        if not exe:
+            yield Severity.ERROR, 'no shebang executable'
+
         if extension in FILE_EXTENSIONS:
             if not FILE_EXTENSIONS[extension].search(exe):
                 yield (
@@ -111,7 +111,7 @@ class CheckExecutable(CommittedFileCheck):
                 return
 
             # We are white-listing general names to have a file extension.
-            name = self.committed_file.get_filename()[:(len(extension) + 1)]
+            name = self.committed_file.get_filename()[:-(len(extension) + 1)]
             if name not in GENERAL_EXECUTABLE_NAMES:
                 yield Severity.WARNING, 'redundant file extension'
             return
@@ -133,19 +133,18 @@ class CheckSymlink(CommittedFileCheck):
     def prepare(self, obj):
         new = super(CheckSymlink, self).prepare(obj)
         if not new or (
-            isinstance(obj, CommittedFile) and not obj.get_symlink_target()
+            isinstance(obj, CommittedFile) and not obj.symlink()
         ):
             return None
         return new
 
     def get_problems(self):
-        symlink_target = self.committed_file.get_symlink_target()
-        target_file = CommittedFile(symlink_target, self.committed_file.commit)
-        if not target_file.exists():
+        target = self.committed_file.get_symlink_target()
+        if not target or not target.exists():
             yield (
                 Severity.WARNING,
-                'symlink target "{}" doesn\'t exist on repository'
-                .format(symlink_target)
+                'symlink target {} doesn\'t exist on repository'
+                .format(target)
             )
 
 
@@ -157,25 +156,27 @@ class CommittedFileByExtensionCheck(CommittedFileCheck):
         if not new or not isinstance(obj, CommittedFile):
             return new
 
-        # There is no point of checking links by their type.
-        if obj.get_symlink_target():
-            return None
-
         # All instances of this must specify a file extension.
         assert new.extension
 
-        # We are being prepared for a committed file as last.  In this step,
+        # There is no point of checking links by their type.
+        if obj.symlink():
+            return None
+
+        # There is no point of checking empty files by their type.
+        if not obj.get_content():
+            return None
+
+        # We are being prepared for a committed file at last.  In this step,
         # we need to match the file with the specified file extension.
         # We first check the extension from the name of the file, and then
-        # from the shebang of the file, if it is executable.
+        # from the shebang of the file.
         if obj.get_extension() == new.extension:
             return new
-        if (
-            new.extension in FILE_EXTENSIONS and
-            obj.owner_can_execute() and
-            FILE_EXTENSIONS[new.extension].search(obj.get_exe())
-        ):
-            return new
+        if new.extension in FILE_EXTENSIONS:
+            exe = obj.get_shebang_exe()
+            if exe and FILE_EXTENSIONS[new.extension].search(exe):
+                return new
 
         return None
 
@@ -240,61 +241,68 @@ class CheckCommand(CommittedFileByExtensionCheck):
         return config_exists
 
     def _prepare_proc(self):
-        self._proc = self.committed_file.pass_content(
-            [self.get_exe_path()] + self.args[1:]
+        self._proc = Popen(
+            [self.get_exe_path()] + self.args[1:],
+            stdin=PIPE,
+            stdout=PIPE,
+            stderr=STDOUT,
         )
+        with self._proc.stdin as fd:
+            fd.write(self.committed_file.get_content())
 
     def get_problems(self):
         line_buffer = []
-        for line_id, line in enumerate(self._proc.stdout):
-            if line_id < self.header:
-                continue
-            line_buffer.append(line)
-            if len(line_buffer) <= self.footer:
-                continue
-            yield self._format_problem(line_buffer.pop(0).strip().decode())
+        with self._proc.stdout as fd:
+            for line_id, line in enumerate(fd):
+                if line_id < self.header:
+                    continue
+                line_buffer.append(line)
+                if len(line_buffer) <= self.footer:
+                    continue
+                yield self._format_problem(line_buffer.pop(0))
 
-        if self._proc.wait() != 0:
-            check_returncode(self._proc.content_proc)
-            if (
-                not self.bogus_return_code and
-                not self.committed_file.commit.content_can_fail()
-            ):
-                self.set_state(CheckState.FAILED)
+        return_code = self._proc.wait()
+        if (
+            return_code != 0 and
+            not self.bogus_return_code and
+            not self.committed_file.commit.content_can_fail()
+        ):
+            self.set_state(CheckState.FAILED)
 
     def _format_problem(self, line):
         """We are piping the source from Git to the commands.  We want to
         hide the file path from the users as we show it already on the headers.
         """
         prefix = ''
+        rest = line.strip().decode('utf-8')
 
-        line_split = line.split(':', 3)
+        rest_split = rest.split(':', 3)
         if (
-            len(line_split) == 4 and
-            len(line_split[0]) >= len('stdin') and
-            line_split[1].isdigit() and
-            line_split[2].isdigit()
+            len(rest_split) == 4 and
+            len(rest_split[0]) >= len('stdin') and
+            rest_split[1].isdigit() and
+            rest_split[2].isdigit()
         ):
-            prefix = 'line {}: col {}: '.format(*line_split[1:3])
-            line = line_split[3].strip()
+            prefix = 'line {}: col {}: '.format(*rest_split[1:3])
+            rest = rest_split[3].strip()
         else:
-            if len(line_split) >= 2 and 'stdin' in line_split[0].lower():
-                line = ':'.join(line_split[1:]).strip()
-            if line.startswith('line '):
-                line_split = line.split(' ', 2)
-                line_num = line_split[1].strip(':,')
+            if len(rest_split) >= 2 and 'stdin' in rest_split[0].lower():
+                rest = ':'.join(rest_split[1:]).strip()
+            if rest.startswith('line '):
+                rest_split = rest.split(' ', 2)
+                line_num = rest_split[1].strip(':,')
                 if line_num.isdigit():
                     prefix += 'line ' + line_num + ': '
-                    line = ' '.join(line_split[2:]).strip(':,')
-            if line.startswith('col '):
-                line_split = line.split(' ', 2)
-                col_num = line_split[1].strip(':,')
+                    rest = ' '.join(rest_split[2:]).strip(':,')
+            if rest.startswith('col '):
+                rest_split = rest.split(' ', 2)
+                col_num = rest_split[1].strip(':,')
                 if col_num.isdigit():
                     prefix += 'col ' + col_num + ': '
-                    line = ' '.join(line_split[2:]).strip(':,')
+                    rest = ' '.join(rest_split[2:]).strip(':,')
 
-        severity, line = Severity.split(line)
-        return severity, prefix + line
+        severity, rest = Severity.split(rest)
+        return severity, prefix + rest
 
     def __str__(self):
         return '{} "{}" on {}'.format(
@@ -316,7 +324,7 @@ class FormatCheck(CommittedFileByExtensionCheck):
     def get_problems(self):
         assert self.load_func and self.exception_cls
         try:
-            self.load_func(self.committed_file.get_content().decode())
+            self.load_func(self.committed_file.get_content().decode('utf-8'))
         except self.exception_cls as error:
             yield Severity.ERROR, str(error)
 
