@@ -1,9 +1,13 @@
+# -*- coding: utf-8 -*-
 """igcommit - Git routines
 
 Copyright (c) 2016, InnoGames GmbH
 """
 
-from subprocess import check_output, CalledProcessError, Popen, PIPE, STDOUT
+from __future__ import unicode_literals
+
+from os.path import isabs, join as joinpath, normpath
+from subprocess import check_output
 
 from igcommit.utils import get_exe_path
 
@@ -33,8 +37,7 @@ class Commit(object):
     def __init__(self, commit_id, commit_list=None):
         self.commit_id = commit_id
         self.commit_list = commit_list
-        self._content_proc = None
-        self._message = None
+        self.content_fetched = False
         self.changed_files = None
 
     def __str__(self):
@@ -58,7 +61,7 @@ class Commit(object):
             '--not',
             '--all',
             '--reverse',
-        ]).decode()
+        ]).decode('utf-8')
         commit_list = CommitList([], ref_path)
         for commit_id in output.splitlines():
             commit = Commit(commit_id, commit_list)
@@ -66,34 +69,39 @@ class Commit(object):
         return commit_list
 
     def _fetch_content(self):
-        self._content_proc = Popen(
-            [git_exe_path, 'cat-file', '-p', self.commit_id],
-            stdout=PIPE,
+        content = check_output(
+            [git_exe_path, 'cat-file', '-p', self.commit_id]
         )
         self._parents = []
+        self._message_lines = []
         # The commit message starts after the empty line.  We iterate until
         # we find one, and then consume the rest as the message.
-        for line in iter(self._content_proc.stdout.readline, b'\n'):
+        lines = iter(content.splitlines())
+        for line in lines:
+            if not line:
+                break
             if line.startswith(b'parent '):
                 self._parents.append(Commit(line[len(b'parent '):].rstrip()))
-            if line.startswith(b'author '):
+            elif line.startswith(b'author '):
                 self._author = Contribution.parse(line[len(b'author '):])
-            if line.startswith(b'committer '):
+            elif line.startswith(b'committer '):
                 self._committer = Contribution.parse(line[len(b'committer '):])
-            check_returncode(self._content_proc)
+        for line in lines:
+            self._message_lines.append(line.decode('utf-8'))
+        self.content_fetched = True
 
     def get_parents(self):
-        if not self._content_proc:
+        if not self.content_fetched:
             self._fetch_content()
         return self._parents
 
     def get_author(self):
-        if not self._content_proc:
+        if not self.content_fetched:
             self._fetch_content()
         return self._author
 
     def get_committer(self):
-        if not self._content_proc:
+        if not self.content_fetched:
             self._fetch_content()
         return self._committer
 
@@ -101,17 +109,13 @@ class Commit(object):
         yield self.get_author()
         yield self._committer
 
-    def get_message(self):
-        if not self._content_proc:
+    def get_message_lines(self):
+        if not self.content_fetched:
             self._fetch_content()
-        if not self._message:
-            self._message = self._content_proc.stdout.read().decode('utf8')
-            check_returncode(self._content_proc)
-        return self._message
+        return self._message_lines
 
     def get_summary(self):
-        for line in self.get_message().splitlines():
-            return line
+        return self.get_message_lines()[0]
 
     def parse_tags(self):
         tags = []
@@ -134,12 +138,13 @@ class Commit(object):
                 git_exe_path,
                 'diff-tree',
                 '-r',
-                '--no-commit-id',
+                '--root',               # Get the initial commit as additions
+                '--no-commit-id',       # We already know the commit id.
                 '--break-rewrites',     # Get rewrites as additions
                 '--no-renames',         # Get renames as additions
                 '--diff-filter=AM',     # Only additions and modifications
                 self.commit_id,
-            ]).decode()
+            ]).decode('utf-8')
             changed_files = []
             for line in output.splitlines():
                 line_split = line.split()
@@ -166,7 +171,7 @@ class Contribution(object):
         name, line = line.split(b' <', 1)
         email, line = line.split(b'> ', 1)
         timestamp, line = line.split(b' ', 1)
-        return cls(name.decode('utf8'), email.decode(), int(timestamp))
+        return cls(name.decode('utf-8'), email.decode('utf-8'), int(timestamp))
 
     def get_email_domain(self):
         return self.email.split('@', 1)[-1]
@@ -180,8 +185,7 @@ class CommittedFile(object):
         self.commit = commit
         assert mode is None or len(mode) == 6
         self.mode = mode
-        self._not_consumed_content_proc = None
-        self._first_content_line = None
+        self.content = None
 
     def __str__(self):
         return '{} at {}'.format(self.path, self.commit)
@@ -206,6 +210,9 @@ class CommittedFile(object):
     def changed(self):
         return self in self.commit.get_changed_files()
 
+    def symlink(self):
+        return self.mode[1] == '2'
+
     def owner_can_execute(self):
         owner_bits = int(self.mode[-3])
         return bool(owner_bits & 1)
@@ -218,108 +225,45 @@ class CommittedFile(object):
             return self.path.rsplit('.', 1)[1]
         return None
 
-    def _spawn_content_proc(self, stdout=PIPE):
-        """Spawn and return git process to get file content"""
-        return Popen((
-            git_exe_path, 'show', self.commit.commit_id + ':' + self.path
-        ), stdout=stdout)
-
-    def _get_first_content_line(self):
-        """Save and return the first line of the file content
-
-        We need the first line of the content for the symlinks and the shebang
-        of the scripts.  We are saving the first line and the process we have
-        used to get the first line to be re-used by the other methods below.
-        """
-        if self._first_content_line is None:
-            assert self._not_consumed_content_proc is None
-            proc = self._spawn_content_proc()
-            self._first_content_line = proc.stdout.readline()
-            check_returncode(proc)
-            self._not_consumed_content_proc = proc
-        return self._first_content_line
-
-    def get_symlink_target(self):
-        """Check if the file is a symlink and return its target"""
-        if self.mode[1] == '2':
-            return self._get_first_content_line().strip()
-        return None
+    def get_content(self):
+        """Get the file content as binary"""
+        if self.content is None:
+            self.content = check_output([
+                git_exe_path, 'show', self.commit.commit_id + ':' + self.path
+            ])
+        return self.content
 
     def get_shebang(self):
         """Get the shebang from the file content"""
-        line = self._get_first_content_line()
-        if line.startswith(b'#!'):
-            return line[len(b'#!'):].decode()
-        return ''
+        content = self.get_content()
+        if content.startswith(b'#!'):
+            return content[len(b'#!'):].split(None, 1)[0].decode('utf-8')
+        return None
 
-    def get_exe(self):
+    def get_shebang_exe(self):
         """Get the executable from the shebang"""
-        shebang_split = self.get_shebang().split(None, 2)
-        if not shebang_split:
-            return ''
-        if shebang_split[0] == '/usr/bin/env' and len(shebang_split) > 1:
-            return shebang_split[1]
-        return shebang_split[0].rsplit('/', 1)[-1]
+        shebang = self.get_shebang()
+        if shebang == '/usr/bin/env':
+            rest = self.get_content().splitlines()[0][len(b'#!/usr/bin/env'):]
+            rest_split = rest.split(None, 1)
+            if rest_split:
+                return rest_split[0].decode('utf-8')
+        elif shebang:
+            return shebang.rsplit('/', 1)[-1]
+        return None
 
-    def get_content(self):
-        """Get the file content as binary
+    def get_symlink_target(self):
+        """Get the symlink target as same kind of instance
 
-        We have two ways of get the content.  If get_shebang() method is
-        called before, we must have a file descriptor which has only the first
-        line consumed.  We can consume the rest and append to the first line.
-        """
-        if self._not_consumed_content_proc is not None:
-            assert self._first_content_line is not None
-            content = self._first_content_line
-            proc = self._not_consumed_content_proc
-            self._not_consumed_content_proc = None
-        else:
-            content = b''
-            proc = self._spawn_content_proc()
-
-        content += proc.communicate()[0]
-        check_returncode(proc)
-        return content
-
-    def pass_content(self, proc_args):
-        """Pass the file content to another process
-
-        We have two ways of passing the content.  If get_shebang() method is
-        called before, we must have a file descriptor which has only the first
-        line consumed.  In this case, we need to read and pass the content
-        by ourselves.  If we don't yet have a file descriptor, we can take
-        the shortcut and just pipe two processes together.
-
-        It is not very nice to have two different ways of doing the same
-        thing, though that is the most efficient method.  We would be wasting
-        the Git content process by only reading the first line of it to get
-        the shebang, if we wouldn't be using it in here.  Likewise things
-        would be slower, if we buffer the content of every file in here
-        to pass their content to the target process.  The shebang is only
-        read for executable files, so most of the files should benefit from
-        the optimization.  This is also an important optimization for
-        parallelization.  The less we do on the main process, the more we can
-        benefit from multiple cores on the system.  Before doing thing this
-        way, I had the cute idea to pass the stdout of the content process
-        we have used to get the shebang to the target process directly.  It
-        was causing the line numbers on the syntax errors to be 1 off.
-        """
-        if self._not_consumed_content_proc is not None:
-            content_proc = self._not_consumed_content_proc
-            stdin = PIPE
-        else:
-            content_proc = self._spawn_content_proc()
-            stdin = content_proc.stdout
-        target_proc = Popen(proc_args, stdin=stdin, stdout=PIPE, stderr=STDOUT)
-        # Someone has to check the output of the content process later.
-        target_proc.content_proc = content_proc
-
-        if self._not_consumed_content_proc is not None:
-            stdin = target_proc.stdin
-            stdin.write(self.get_content())
-        stdin.close()   # Allow it to receive a SIGPIPE
-
-        return target_proc
+        We just return None, if the target has no chance to be on
+        the repository."""
+        content = self.get_content()
+        if isabs(content):
+            return None
+        path = normpath(joinpath(self.path, '..', content.decode('utf-8')))
+        if path.startswith('..'):
+            return None
+        return type(self)(path, self.commit)
 
     def write(self):
         """Write the file contents to the location its supposed to be
@@ -327,20 +271,7 @@ class CommittedFile(object):
         The Git pre-receive hooks can work on bare Git repositories.  Those
         repositories has no actual files, only the .git database.  We need
         to materialize the configuration files on their locations for
-        the syntax checking tools to find them.  We are doing it efficiently
-        by passing the writable file descriptor to the Git content process.
+        the syntax checking tools to find them.
         """
         with open(self.path, 'wb') as fd:
-            proc = self._spawn_content_proc(fd)
-        proc.wait()
-        check_returncode(proc)
-
-
-def check_returncode(proc):
-    if proc.returncode is None:
-        proc.poll()
-        if proc.returncode is None:
-            return
-
-    if proc.returncode != 0:
-        raise CalledProcessError(proc.returncode, proc.args)
+            fd.write(self.get_content())
